#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
uint8_t in(uint8_t pin)
{
	if(pin!=6){
    		DDRC &= ~(1<<pin);
			PORTC |= 1<<pin;
			return (PINC >>pin) & 1;
	}else{
			DDRD &= ~(1<<0);
			PORTD |= 1<<0;
			return (PIND >>0) & 1;
	}
}

void out(uint8_t pin, uint8_t val)
{
	if (val)
		if(pin!=6){
			DDRB |= 1<<pin;
			PORTB |= 1<<pin;			
		}else{
			DDRD |= 1<<5;
			PORTD |= 1<<5;
		}
	else
		if(pin!=6){
			DDRB &= ~(1<<pin);			
			PORTB &= ~(1<<pin);			
		}else{
			DDRD &= ~(1<<5);
			PORTD &= ~(1<<5);
		}
	
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

static uint8_t inSwitch;
usbMsgLen_t usbFunctionSetup(uint8_t data[8]){
	usbRequest_t *rq=(void*)data;
	static uint8_t dataBuffer[1];
	if (rq->bRequest == VENDOR_RQ_SET_LED)
	{
		/*
		uint8_t led_no  = rq->wValue.bytes[0];
		uint8_t led_val = rq->wValue.bytes[1];
		*/
		return 0;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
	{
		dataBuffer[0] = inSwitch; /* กลับลอจิกเพื่อให้ 0 = ปล่อย 1 = กด */
		usbMsgPtr = dataBuffer;   /* ระบุว่าข้อมูลส่งกลับอยู่ใน dataBuffer */
		return 1;                 /* ความยาวข้อมูลส่งกลับเท่ากับ 1 */
	}

}

/* ------------------------------------------------------------------------- */

int main(void)
{
	uint8_t i;

	DDRB=0b00111111;
	DDRC=0b00000000;
	DDRD=0b00100000;
		
	usbInit();
	usbDeviceDisconnect();
	_delay_ms(300);
	usbDeviceConnect();
	sei();
	
	while(1){               /* main event loop */
		usbPoll();
		for(i=0;i<=6;i++)
			if(!in(i)){
				inSwitch|=1<<i;
				out(i,1);
			}else{
				inSwitch&=~(1<<i);
				out(i,0);
			}		
	}
    	return 0;
}

/* ------------------------------------------------------------------------- */
